@@ -162,14 +162,14 @@ fs_rawudp_transmitter_register_type (FsPlugin *module)
 
   fs_rawudp_stream_transmitter_register_type (module);
 
-  type = g_type_register_static (FS_TYPE_TRANSMITTER, "FsRawUdpTransmitter",
-      &info, 0);
+  type = g_type_module_register_type (G_TYPE_MODULE (module),
+      FS_TYPE_TRANSMITTER, "FsRawUdpTransmitter", &info, 0);
 
   return type;
 }
 
 
-FS_INIT_PLUGIN (rawudp, transmitter)
+FS_INIT_PLUGIN (fs_rawudp_transmitter_register_type)
 
 static void
 fs_rawudp_transmitter_class_init (FsRawUdpTransmitterClass *klass)
@@ -531,6 +531,10 @@ struct _UdpPort {
   GstElement *udpsink;
   GstPad *udpsink_requested_pad;
 
+  GstElement *recvonly_filter;
+  GstElement *recvonly_udpsink;
+  GstPad *recvonly_requested_pad;
+
   gchar *requested_ip;
   guint requested_port;
 
@@ -889,6 +893,19 @@ fs_rawudp_transmitter_get_udpport (FsRawUdpTransmitter *trans,
   if (!udpport->udpsink)
     goto error;
 
+  udpport->recvonly_filter = fs_transmitter_get_recvonly_filter (
+      FS_TRANSMITTER (trans), udpport->component_id);
+
+  if (udpport->recvonly_filter)
+  {
+    udpport->recvonly_udpsink = _create_sinksource ("multiudpsink",
+        GST_BIN (trans->priv->gst_sink), udpport->tee,
+        udpport->recvonly_filter, udpport->socket, GST_PAD_SINK, FALSE,
+        &udpport->recvonly_requested_pad, error);
+    if (!udpport->recvonly_udpsink)
+      goto error;
+  }
+
   g_mutex_lock (&trans->priv->mutex);
 
   /* Check if someone else added the same port at the same time */
@@ -971,6 +988,27 @@ fs_rawudp_transmitter_put_udpport (FsRawUdpTransmitter *trans,
       GST_ERROR ("Could not remove udpsink element from transmitter source");
   }
 
+  if (udpport->recvonly_requested_pad)
+  {
+    gst_element_release_request_pad (udpport->tee,
+        udpport->recvonly_requested_pad);
+    gst_object_unref (udpport->recvonly_requested_pad);
+  }
+
+  if (udpport->recvonly_udpsink)
+  {
+    GstStateChangeReturn ret;
+    gst_element_set_locked_state (udpport->recvonly_udpsink, TRUE);
+    ret = gst_element_set_state (udpport->recvonly_udpsink, GST_STATE_NULL);
+    if (ret != GST_STATE_CHANGE_SUCCESS)
+      GST_ERROR ("Error changing state of udpsink: %s",
+          gst_element_state_change_return_get_name (ret));
+    if (!gst_bin_remove (GST_BIN (trans->priv->gst_sink),
+            udpport->recvonly_udpsink))
+      GST_ERROR ("Could not remove udpsink element from transmitter source");
+  }
+
+
   if (udpport->socket)
     g_socket_close (udpport->socket, NULL);
   g_clear_object (&udpport->socket);
@@ -1220,6 +1258,25 @@ fs_rawudp_transmitter_udpport_remove_known_address (UdpPort *udpport,
   g_mutex_unlock (&udpport->mutex);
 }
 
+void
+fs_rawudp_transmitter_udpport_add_recvonly_dest (UdpPort *udpport,
+    const gchar *ip,
+    gint port)
+{
+  if (udpport->recvonly_udpsink)
+    g_signal_emit_by_name (udpport->recvonly_udpsink, "add", ip, port);
+}
+
+
+void
+fs_rawudp_transmitter_udpport_remove_recvonly_dest (UdpPort *udpport,
+    const gchar *ip,
+    gint port)
+{
+  if (udpport->recvonly_udpsink)
+    g_signal_emit_by_name (udpport->recvonly_udpsink, "remove", ip, port);
+}
+
 static void
 fs_rawudp_transmitter_set_type_of_service (FsRawUdpTransmitter *self,
     gint tos)
